/* eslint class-methods-use-this:0 */
// @ts-check
'use strict'; // Import

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (typeof call === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

var _require = require('caterpillar'),
    Transform = _require.Transform;

var util = require('util');

var ansiColors, ansiStyles;

try {
  ansiColors = require('ansicolors');
  ansiStyles = require('ansistyles');
} catch (err) {
  ansiColors = null;
  ansiStyles = null;
}
/**
Return the given argument.
Used for when there is no formatter.
@private
@param {string} str
@returns {string}
*/


function nop(str) {
  return str;
}
/**
Convert Logger entries into human readable format.
Extends http://rawgit.com/bevry/caterpillar/master/docs/index.html#Transform
@extends Transform
@example
const logger = require('caterpillar').create()
logger.pipe(require('caterpillar-human').create()).pipe(process.stdout)
logger.log('info', 'some', {data: 'oh yeah'}, 42)
*/


var Human =
/*#__PURE__*/
function (_Transform) {
  _inherits(Human, _Transform);

  function Human() {
    _classCallCheck(this, Human);

    return _possibleConstructorReturn(this, _getPrototypeOf(Human).apply(this, arguments));
  }

  _createClass(Human, [{
    key: "getInitialConfig",

    /**
    Get the initial configuration.
    @returns {Object}
    */
    value: function getInitialConfig() {
      return {
        color: true,
        level: null,
        colors: {
          '0': 'red',
          '1': 'red',
          '2': 'red',
          '3': 'red',
          '4': 'yellow',
          '5': 'yellow',
          '6': 'green',
          '7': 'green'
        }
      };
    }
    /**
    Get the color for the log level
    @param {number} levelNumber
    @returns {string}
    */

  }, {
    key: "getColor",
    value: function getColor(levelNumber) {
      // Determine
      var _this$getConfig = this.getConfig(),
          colors = _this$getConfig.colors;

      var color = colors[levelNumber] || false; // Return

      return color;
    }
    /**
    Pad the left of some content if need be with the specified padding to make the content reach a certain size
    @param {string} padding
    @param {number} size
    @param {string|number} content
    @returns {string}
    */

  }, {
    key: "padLeft",
    value: function padLeft(padding, size, content) {
      // Prepare
      padding = String(padding);
      content = String(content); // Handle

      if (content.length < size) {
        for (var i = 0, n = size - content.length; i < n; ++i) {
          content = padding + content;
        }
      } // Return


      return content;
    }
    /**
    Convert logger entry arguments into a human readable string
    @param {Array<any>} args
    @returns {string}
    */

  }, {
    key: "formatArguments",
    value: function formatArguments(args) {
      var _this$getConfig2 = this.getConfig(),
          color = _this$getConfig2.color;

      return args.map(function (value) {
        return typeof value === 'string' ? value : util.inspect(value, {
          showHidden: false,
          depth: 10,
          colors: color
        });
      }).join(' ');
    }
    /**
    Convert a datetime into a human readable format
    @param {Date|number|string} datetime
    @returns {string}
    */

  }, {
    key: "formatDate",
    value: function formatDate(datetime) {
      // Prepare
      var now = new Date(datetime);
      var year = now.getFullYear();
      var month = this.padLeft('0', 2, now.getMonth() + 1);
      var date = this.padLeft('0', 2, now.getDate());
      var hours = this.padLeft('0', 2, now.getHours());
      var minutes = this.padLeft('0', 2, now.getMinutes());
      var seconds = this.padLeft('0', 2, now.getSeconds());
      var ms = this.padLeft('0', 3, now.getMilliseconds()); // Apply

      var result = "".concat(year, "-").concat(month, "-").concat(date, " ").concat(hours, ":").concat(minutes, ":").concat(seconds, ".").concat(ms); // Return

      return result;
    }
    /**
    Convert a logger entry into a human readable format
    @param {string} message
    @returns {string}
    */

  }, {
    key: "format",
    value: function format(message) {
      // Prepare
      var entry = JSON.parse(message);

      var _this$getConfig3 = this.getConfig(),
          color = _this$getConfig3.color,
          level = _this$getConfig3.level;

      var debug = level === 7;
      var result = null; // Format

      entry.color = this.getColor(entry.levelNumber);
      entry.timestamp = this.formatDate(entry.date);
      entry.text = this.formatArguments(entry.args); // Check

      if (entry.text) {
        // Formatters
        var levelFormatter = color && (ansiColors && ansiColors[entry.color] || ansiStyles && ansiStyles[entry.color]) || nop;
        var textFormatter = false && debug && color && ansiStyles && ansiStyles.bright || nop;
        var debugFormatter = debug && color && ansiStyles && ansiStyles.dim || nop; // Message

        var levelString = levelFormatter("".concat(entry.levelName, ":"));
        var entryString = textFormatter(entry.text);
        var messageString = "".concat(levelString, " ").concat(entryString); // Debugging

        if (debug) {
          // Debug Information
          var seperator = '\n    ';
          var debugString = debugFormatter("\u2192 [".concat(entry.timestamp, "] [").concat(entry.file, ":").concat(entry.line, "] [").concat(entry.method, "]")); // Result

          result = "".concat(messageString).concat(seperator).concat(debugString, "\n");
        } else {
          // Result
          result = "".concat(messageString, "\n");
        }
      } // @TODO should we throw on else?
      // Return


      return result;
    }
  }]);

  return Human;
}(Transform); // Export


module.exports = Human;